@@ -1,4 +1,5 @@
 import enum
+
 import zigpy.types
 
 
@@ -15,7 +16,7 @@ def deserialize(data, schema):
 
 
 def serialize(data, schema):
-    return b''.join(t(v).serialize() for t, v in zip(schema, data))
+    return b"".join(t(v).serialize() for t, v in zip(schema, data))
 
 
 class Bytes(bytes):
@@ -24,7 +25,7 @@ class Bytes(bytes):
 
     @classmethod
     def deserialize(cls, data):
-        return cls(data), b''
+        return cls(data), b""
 
 
 class LBytes(bytes):
@@ -32,25 +33,24 @@ class LBytes(bytes):
         return uint8_t(len(self)).serialize() + self
 
     @classmethod
-    def deserialize(cls, data, byteorder='big'):
+    def deserialize(cls, data, byteorder="big"):
         _bytes = int.from_bytes(data[:1], byteorder)
-        s = data[1:_bytes + 1]
-        return s, data[_bytes + 1:]
+        s = data[1 : _bytes + 1]
+        return s, data[_bytes + 1 :]
 
 
 class int_t(int):
     _signed = True
     _size = 0
 
-    def serialize(self, byteorder='big'):
+    def serialize(self, byteorder="big"):
         return self.to_bytes(self._size, byteorder, signed=self._signed)
 
     @classmethod
-    def deserialize(cls, data, byteorder='big'):
+    def deserialize(cls, data, byteorder="big"):
         # Work around https://bugs.python.org/issue23640
-        r = cls(int.from_bytes(data[:cls._size],
-                               byteorder, signed=cls._signed))
-        data = data[cls._size:]
+        r = cls(int.from_bytes(data[: cls._size], byteorder, signed=cls._signed))
+        data = data[cls._size :]
         return r, data
 
 
@@ -144,9 +144,20 @@ class NWK(uint16_t):
 class AddressMode(uint8_t, enum.Enum):
     # Address modes used in zigate protocol
 
+    BOUND = 0x00
     GROUP = 0x01
     NWK = 0x02
     IEEE = 0x03
+    BROADCAST = 0x04
+
+    NO_TRANSMIT = 0x05
+
+    BOUND_NO_ACK = 0x06
+    NWK_NO_ACK = 0x07
+    IEEE_NO_ACK = 0x08
+
+    BOUND_NON_BLOCKING = 0x09
+    BOUND_NON_BLOCKING_NO_ACK = 0x0A
 
 
 class Status(uint8_t, enum.Enum):
@@ -196,6 +207,52 @@ class Status(uint8_t, enum.Enum):
     # No free BTR entries left.
     NoFreeBTREntries = 0x8B
 
+    # A transmit request failed since the ASDU is too large and fragmentation is not
+    # supported.
+    AsduTooLong = 0xA0
+    # A received fragmented frame could not be defragmented at the current time.
+    DefragDeferred = 0xA1
+    # A received fragmented frame could not be defragmented since the device does not
+    # support fragmentation.
+    DefragUnsupported = 0xA2
+    # A parameter value was out of range.
+    IllegalRequest = 0xA3
+    # An APSME-UNBIND.request failed due to the requested binding link not existing in
+    # the binding table.
+    InvalidBinding = 0xA4
+    # An APSME-REMOVE-GROUP.request has been issued with a group identifier that does
+    # not appear in the group table.
+    InvalidGroup = 0xA5
+    # A parameter value was invalid or out of range.
+    InvalidParameter = 0xA6
+    # An APSDE-DATA.request requesting acknowledged transmission failed due to no
+    # acknowledgement being received.
+    NoAck = 0xA7
+    # An APSDE-DATA.request with a destination addressing mode set to 0x00 failed due to
+    # there being no devices bound to this device.
+    NoBoundDevice = 0xA8
+    # An APSDE-DATA.request with a destination addressing mode set to 0x03 failed due to
+    # no corresponding short address found in the address map table.
+    NoShortAddress = 0xA9
+    # An APSDE-DATA.request with a destination addressing mode set to 0x00 failed due to
+    # a binding table not being supported on the device.
+    NotSupported = 0xAA
+    # An ASDU was received that was secured using a link key.
+    SecuredLinkKey = 0xAB
+    # An ASDU was received that was secured using a network key.
+    SecuredNwkKey = 0xAC
+    # An APSDE-DATA.request requesting security has resulted in an error during the
+    # corresponding security processing.
+    SecurityFail = 0xAD
+    # An APSME-BIND.request or APSME.ADDGROUP.request issued when the binding or group
+    # tables, respectively, were full.
+    TableFull = 0xAE
+    # An ASDU was received without any security.
+    Unsecured = 0xAF
+    # An APSME-GET.request or APSMESET. request has been issued with an unknown
+    # attribute identifier.
+    UnsupportedAttribute = 0xB0
+
     @classmethod
     def _missing_(cls, value):
         if not isinstance(value, int):
@@ -236,7 +293,7 @@ class Struct:
                 setattr(self, k, v)
 
     def serialize(self):
-        r = b''
+        r = b""
         for field in self._fields:
             if hasattr(self, field[0]):
                 r += getattr(self, field[0]).serialize()
@@ -251,18 +308,37 @@ class Struct:
         return r, data
 
     def __repr__(self):
-        r = '<%s ' % (self.__class__.__name__, )
-        r += ' '.join(
-            ['%s=%s' % (f[0], getattr(self, f[0], None)) for f in self._fields]
+        r = "<{} ".format(self.__class__.__name__)
+        r += " ".join(
+            ["{}={}".format(f[0], getattr(self, f[0], None)) for f in self._fields]
         )
-        r += '>'
+        r += ">"
         return r
 
 
+ZIGPY_TO_ZIGATE_ADDR_MODE = {
+    # With ACKs
+    (zigpy.types.AddrMode.NWK, True): AddressMode.NWK,
+    (zigpy.types.AddrMode.IEEE, True): AddressMode.IEEE,
+    (zigpy.types.AddrMode.Broadcast, True): AddressMode.BROADCAST,
+    (zigpy.types.AddrMode.Group, True): AddressMode.GROUP,
+    # Without ACKs
+    (zigpy.types.AddrMode.NWK, False): AddressMode.NWK_NO_ACK,
+    (zigpy.types.AddrMode.IEEE, False): AddressMode.IEEE_NO_ACK,
+    (zigpy.types.AddrMode.Broadcast, False): AddressMode.BROADCAST,
+    (zigpy.types.AddrMode.Group, False): AddressMode.GROUP,
+}
+
+ZIGATE_TO_ZIGPY_ADDR_MODE = {
+    zigate_addr: (zigpy_addr, ack)
+    for (zigpy_addr, ack), zigate_addr in ZIGPY_TO_ZIGATE_ADDR_MODE.items()
+}
+
+
 class Address(Struct):
     _fields = [
-        ('address_mode', AddressMode),
-        ('address', EUI64),
+        ("address_mode", AddressMode),
+        ("address", EUI64),
     ]
 
     def __eq__(self, other):
@@ -271,17 +347,25 @@ class Address(Struct):
     @classmethod
     def deserialize(cls, data):
         r = cls()
-        field_name, field_type = cls._fields[0]
-        mode, data = field_type.deserialize(data)
-        setattr(r, field_name, mode)
-        v = None
-        if mode in [AddressMode.GROUP, AddressMode.NWK]:
-            v, data = NWK.deserialize(data)
-        elif mode == AddressMode.IEEE:
-            v, data = EUI64.deserialize(data)
-        setattr(r, cls._fields[1][0], v)
+        r.address_mode, data = AddressMode.deserialize(data)
+
+        if r.address_mode in (AddressMode.IEEE, AddressMode.IEEE_NO_ACK):
+            r.address, data = EUI64.deserialize(data)
+        else:
+            r.address, data = NWK.deserialize(data)
+
         return r, data
 
+    def to_zigpy_type(self):
+        zigpy_addr_mode, ack = ZIGATE_TO_ZIGPY_ADDR_MODE[self.address_mode]
+
+        return (
+            zigpy.types.AddrModeAddress(
+                addr_mode=zigpy_addr_mode, address=self.address
+            ),
+            ack,
+        )
+
 
 class DeviceEntry(Struct):
     _fields = [
